# coding: utf-8
from nwpc_hpc_model.base.query_model import QueryModel
from nwpc_hpc_model.disk_usage.pi import QueryItem


class PiDiskUsageQueryModel(QueryModel):
    def __init__(self):
        QueryModel.__init__(self)

    @classmethod
    def build_from_category_list(cls, record, category_list):
        if len(record) == 0:
            return None
        user_line = record[0]
        title_line = record[1]
        lines = record[2:]
        model = PiDiskUsageQueryModel()

        for line in lines:
            if len(line) == 0:
                continue
            item = QueryItem.build_from_category_list(line, category_list)
            model.items.append(item)

        return model



# coding=utf-8
from nwpc_hpc_model.base import query_category, query_item, query_property, record_parser, value_saver
from nwpc_hpc_model.base.query_property import QueryProperty
from nwpc_hpc_model.base.query_item import QueryItem
from .query_model import PiDiskUsageQueryModel
from .query_category import PiDiskUsageCategoryList, QueryCategory


# coding: utf-8
from pathlib import Path
import yaml

from nwpc_hpc_model.disk_usage.aix import AixDiskUsageQueryModel, QueryCategory, \
    AixDiskUsageCategoryList, record_parser, value_saver


class TestQuotaInfo(object):
    def check_build_model(self, test_case):
        record = test_case['record']
        category_list = test_case['category_list']
        records = test_case['records']

        model = AixDiskUsageQueryModel.build_from_category_list(record, category_list)

        for record in records:
            index = record['index']
            properties = record['properties']
            item = model.items[index]

            for a_prop in properties:
                category_id = a_prop['category_id']
                data = a_prop['data']
                category_index = category_list.index_from_id(category_id)
                p = item.props[category_index]

                assert(p.map['data'] == data)

    # def test_build_model(self):
    #     check_method = self.check_build_model
    #
    #     category_list = PiDiskUsageCategoryList()
    #
    #     category_list.extend([
    #         QueryCategory("disk_usage.dir", "Direcotry", "dir",
    #                       record_parser.TokenRecordParser, (0,),
    #                       value_saver.StringSaver, ()),
    #         QueryCategory("disk_usage.total_size", "Size", "Size(GB)",
    #                       record_parser.TokenRecordParser, (1,),
    #                       value_saver.NumberSaver, ()),
    #         QueryCategory("disk_usage.used_size", "Used", "Used(GB)",
    #                       record_parser.TokenRecordParser, (2,),
    #                       value_saver.NumberSaver, ()),
    #         QueryCategory("disk_usage.avail_size", "Avail", "Avail(GB)",
    #                       record_parser.TokenRecordParser, (3,),
    #                       value_saver.NumberSaver, ()),
    #         QueryCategory("disk_usage.used_percent", "Used Percent", "Use%",
    #                       record_parser.TokenRecordParser, (4,),
    #                       value_saver.NumberSaver, ()),
    #         QueryCategory("disk_usage.type", "Type", "Type",
    #                       record_parser.TokenRecordParser, (4,),
    #                       value_saver.StringSaver, ()),
    #     ])
    #
    #     test_case_list = list()
    #
    #     nwp_output = Path(
    #         Path(__file__).parent,
    #         "../../data/pi/quotainfo_nwp.txt"
    #     )
    #
    #     with open(nwp_output) as nwp_output_file:
    #         record = nwp_output_file.readlines()
    #         test_case_list.append({
    #             'record': record,
    #             'category_list': category_list,
    #             'records': [
    #                 {
    #                     'index': 0,
    #                     'properties': [
    #                         {
    #                             'category_id': 'disk_usage.dir',
    #                             'data': '/g1'
    #                         },
    #                         {
    #                             'category_id': 'disk_usage.total_size',
    #                             'data': 20.0
    #                         },
    #                         {
    #                             'category_id': 'disk_usage.used_size',
    #                             'data': 1.23
    #                         }
    #                     ]
    #                 },
    #                 {
    #                     'index': 1,
    #                     'properties': [
    #                         {
    #                             'category_id': 'disk_usage.dir',
    #                             'data': '/g2'
    #                         },
    #                         {
    #                             'category_id': 'disk_usage.total_size',
    #                             'data': 900000.0
    #                         },
    #                         {
    #                             'category_id': 'disk_usage.used_size',
    #                             'data': 49788.64
    #                         }
    #                     ]
    #                 }
    #             ]
    #         })
    #
    #     for a_test_case in test_case_list:
    #         check_method(a_test_case)

    def test_build_model_using_config_file(self):
        check_method = self.check_build_model

        config_file_path = Path(
            Path(__file__).parent,
            "../../../../nwpc_hpc_model/disk_usage/aix/conf/disk_usage.config.yml"
        )

        with open(config_file_path) as f:
            category_list_config = yaml.load(f)

        category_list = AixDiskUsageCategoryList.build_from_config(category_list_config['category_list'])

        test_case_list = list()

        nwp_output = Path(
            Path(__file__).parent,
            "../../data/aix/cmquota_nwp.txt"
        )

        with open(nwp_output) as nwp_output_file:
            record = nwp_output_file.readlines()
            test_case_list.append({
                'record': record,
                'category_list': category_list,
                'records': [
                    {
                        'index': 0,
                        'properties': [
                            {
                                'category_id': 'disk_usage.file_system',
                                'data': 'cma_g1'
                            },
                            {
                                'category_id': 'disk_usage.block_limits.current',
                                'data': 28616415232
                            },
                            {
                                'category_id': 'disk_usage.block_limits.quota',
                                'data': 32212254720
                            },
                            {
                                'category_id': 'disk_usage.file_limits.quota',
                                'data': 0
                            }
                        ]
                    },
                    {
                        'index': 1,
                        'properties': [
                            {
                                'category_id': 'disk_usage.file_system',
                                'data': 'cma_u'
                            },
                            {
                                'category_id': 'disk_usage.block_limits.current',
                                'data': 111307008
                            },
                            {
                                'category_id': 'disk_usage.block_limits.quota',
                                'data': 209715200
                            }
                        ]
                    }
                ]
            })

        for a_test_case in test_case_list:
            check_method(a_test_case)




# coding: utf-8
from nwpc_hpc_model.base import query_category, query_item, query_property, value_saver, record_parser
from nwpc_hpc_model.base.query_property import QueryProperty
from nwpc_hpc_model.base.query_item import QueryItem
from .query_model import PiDiskSpaceQueryModel
from .query_category import PiDiskSpaceCategoryList, QueryCategory



# coding=utf-8
from .query_property import QueryProperty


class QueryItem(object):
    def __init__(self):
        self.props = list()

    def to_dict(self):
        result = dict()
        result['props'] = list()
        for prop in self.props:
            result['props'].append(prop.to_dict())
        return result

    @classmethod
    def build_from_category_list(cls, record, category_list):
        item = QueryItem()

        for a_category in category_list:
            p = QueryProperty.build_from_category(record, a_category)
            item.props.append(p)

        return item


def get_property_data(job_item: dict or QueryItem, property_id):
    """
    get property data from a job item
    :param job_item: QueryItem or a QueryItem dict.
    :param property_id: property_id
    :return: property data
    """
    result = None
    if isinstance(job_item, dict):
        props = job_item['props']
        for a_prop in props:
            if a_prop['id'] == property_id:
                result = a_prop['data']
                return result
    else:
        props = job_item.props
        for a_prop in props:
            if a_prop.category.id == property_id:
                result = a_prop.map['data']
                return result
    return result



# coding: utf-8
from pathlib import Path
import yaml

from nwpc_hpc_model.disk_space.aix import AixDiskSpaceQueryModel, QueryCategory, \
    AixDiskSpaceCategoryList, record_parser, value_saver


class TestQuotaInfo(object):
    def check_build_model(self, test_case):
        record = test_case['record']
        category_list = test_case['category_list']
        records = test_case['records']

        model = AixDiskSpaceQueryModel.build_from_category_list(record, category_list)

        for record in records:
            index = record['index']
            properties = record['properties']
            item = model.items[index]

            for a_prop in properties:
                category_id = a_prop['category_id']
                data = a_prop['data']
                category_index = category_list.index_from_id(category_id)
                p = item.props[category_index]

                assert(p.map['data'] == data)

    def test_build_model_using_config_file(self):
        check_method = self.check_build_model

        config_file_path = Path(
            Path(__file__).parent,
            "../../../../nwpc_hpc_model/disk_space/aix/conf/disk_space.config.yml"
        )

        with open(config_file_path) as f:
            category_list_config = yaml.load(f)

        category_list = AixDiskSpaceCategoryList.build_from_config(category_list_config['category_list'])

        test_case_list = list()

        nwp_output = Path(
            Path(__file__).parent,
            "../../data/aix/df_g.txt"
        )

        with open(nwp_output) as nwp_output_file:
            record = nwp_output_file.readlines()
            test_case_list.append({
                'record': record,
                'category_list': category_list,
                'records': [
                    {
                        'index': 10,
                        'properties': [
                            {
                                'category_id': 'disk_space.mounted_on',
                                'data': '/cma/g1'
                            },
                            {
                                'category_id': 'disk_space.blocks.total_size_gb',
                                'data': 268115.86
                            },
                            {
                                'category_id': 'disk_space.blocks.free_size_gb',
                                'data': 86573.93
                            },
                        ]
                    },
                    {
                        'index': 11,
                        'properties': [
                            {
                                'category_id': 'disk_space.mounted_on',
                                'data': '/cma/g2'
                            },
                            {
                                'category_id': 'disk_space.blocks.total_size_gb',
                                'data': 241304.06
                            },
                            {
                                'category_id': 'disk_space.blocks.free_size_gb',
                                'data': 10965.59
                            },
                        ]
                    }
                ]
            })

        for a_test_case in test_case_list:
            check_method(a_test_case)




# coding: utf-8
import re

from nwpc_hpc_model.base.query_model import QueryModel
from . import QueryItem


class PiDiskSpaceQueryModel(QueryModel):
    def __init__(self):
        QueryModel.__init__(self)

    @classmethod
    def build_from_category_list(cls, record, category_list):
        if len(record) == 0:
            return None
        title_line = record[0]
        lines = record[1:]
        model = PiDiskSpaceQueryModel()

        detail_pattern = r'^(\S+) +(\S+) +(\S+) +(\S+) +(\S+) +(\S+)'
        detail_prog = re.compile(detail_pattern)

        for line in lines:
            if len(line) == 0:
                continue

            detail_re_result = detail_prog.match(line)
            if not detail_re_result:
                continue

            item = QueryItem.build_from_category_list(detail_re_result, category_list)
            model.items.append(item)

        return model



# coding: utf-8
from nwpc_hpc_model.base.query_category import QueryCategoryList, QueryCategory
from . import record_parser, value_saver


class PiDiskUsageCategoryList(QueryCategoryList):
    def __init__(self):
        QueryCategoryList.__init__(self)

    @classmethod
    def build_from_config(cls, category_list_config):
        category_list = PiDiskUsageCategoryList()
        for an_item in category_list_config:
            category = QueryCategory(
                category_id=an_item['id'],
                display_name=an_item['display_name'],
                label=an_item['label'],
                record_parser_class=getattr(record_parser, an_item['record_parser_class']),
                record_parser_arguments=tuple(an_item['record_parser_arguments']),
                value_saver_class=getattr(value_saver, an_item['value_saver_class']),
                value_saver_arguments=tuple(an_item['value_saver_arguments'])
            )
            category_list.append(category)
        return category_list



# coding: utf-8
from nwpc_hpc_model.base.query_property import QueryProperty
from nwpc_hpc_model.base.query_model import QueryModel
from nwpc_hpc_model.base.query_item import QueryItem
from nwpc_hpc_model.base.query_category import QueryCategory, QueryCategoryList
